//************************************************************************
// Declaration of the Arrange filter class.
//------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//
//************************************************************************
#ifndef ARRANGE_H_
#define ARRANGE_H_

#include "filter.h"
#include "pairfact.h"
#include "jstring.h"
#include "release.h"

typedef int ArrangeInt;

//************************************************************************
// Class declarations.
//************************************************************************

//------------------------------------------------------------------------
// Class Arrange. In this class a 'pair' refers to an Extractor-Overlayer
// pair: the former extracts part of the input line and the latter
// overlays it onto the result string.
//------------------------------------------------------------------------
class Arrange: public SingleFilter
   {
   public:
      Arrange(const String &args);
      Arrange(int argCount, char **args);
      Arrange(const Arrange &f);
      ~Arrange();

      void processLine(const String &line);
      void minimize();

      DEFINE_FILTER_COPIER(Arrange);
   protected:
      void afterInput();
   private:
      Arrange &operator =(const Arrange &f);

      boolean buildPairs(const String &args, String *errMsg);
      void    allocatePairArrays(ArrangeInt pairCount);
      void    deallocatePairArrays(ArrangeInt pairCount);

      Extractor  **extractor;
      Overlayer  **overlayer;
      ArrangeInt numPairs;

      StringBuffer outputBuffer;

   }; // class Arrange

#ifdef ENABLE_INLINES
   #include "arrange.inl"
#endif

#endif   // #ifndef ARRANGE_H_

