//************************************************************************
// Does a full-coverage test of most of the the arrange filter class`
// code.
//------------------------------------------------------------------------
// Parts of elementsNonNull(), a function are not exercised by this program. 
// Those parts have been tested 'manually'.
//------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//
//************************************************************************

#include <fstream.h>
#include <new.h>

#include "arrange.h"
#include "append.h"
#include "writeto.h"

#include "jstring.h"
#include "release.h"

//------------------------------------------------------------------------
// These strings must be exactly the same as the ones in pairfact.cpp.
//------------------------------------------------------------------------
#define MISSING_BOTH_RANGES_MSG \
   "an input range and an output range must follow the last argument."
#define IS_INVALID_INPUT_RANGE_MSG \
   " is an invalid input range."
#define MISSING_OUTPUT_RANGE_MSG    \
   "an output range must follow the last argument."
#define IS_INVALID_OUTPUT_RANGE_MSG \
   " is an invalid output range."

void test1();
void test2();
void test3();
void test4();
void test5();
void test6();
void test7();
void test8();
void test9();
void test10();

const char outputFile[] = "output.tst";
ofstream outStream(outputFile);

//------------------------------------------------------------------------
// Arranges str using an Arrange filter constructed from the arrangeArgs
// and outputs the result to the outputStream.
//------------------------------------------------------------------------
void arrangeString(const String &str, const String &arrangeArgs,
                   ostream &outputStream)
   {
   String output;

   Append  in  = Append::string(str);
   WriteTo out = WriteTo::string(output, WriteTo::OVERWRITE);

   // using a Filter here allows us to test Arrange`s copy constructor
   // and that of each of its extractors and overlayers
   Filter arr = Arrange(arrangeArgs);
   arr.connect(in, out);
   outputStream << output;
   }

//------------------------------------------------------------------------
// Function called when new fails.
//------------------------------------------------------------------------
void onNewFailure()
   {
	outStream << "Out of memory - exiting." << endl;
   exit(-1);
	}

//------------------------------------------------------------------------
// main().
//------------------------------------------------------------------------
int main()
   {
   set_new_handler(&onNewFailure);

   test1();
   test2();
   test3();
   test4();
   test5();
   test6();
   test7();
   test8();
   test9();
   test10();

   return 0;
   }

//------------------------------------------------------------------------
// #1. Basic tests of the arrange filter. 
//------------------------------------------------------------------------
void test1()
   {
   String input1 = "Hi there!!";

   outStream << "#1" << endl;

   arrangeString(input1, "", outStream);
   outStream << endl;
   arrangeString(input1, "1 2 2 1", outStream);
   outStream << endl;
   arrangeString(input1, "w 2 1 w 1 nextword", outStream);
   outStream << endl;
   arrangeString(input1, "w 2 next", outStream);
   outStream << endl;
   } // test1()

//------------------------------------------------------------------------
// #2. More basic tests, this time on multiple lines.
//------------------------------------------------------------------------
void test2()
   {
   // Note: 17 lines in input1; last line does NOT end with a newline	
   String input1 =  
	   "-rw-r--r--   1 jmackay  users       50072 Jun  2 21:23 arrtest.o\n"
      "-rw-r--r--   1 jmackay  users          44 Jun  2 20:57 arrtest.val\n"
      "-rw-r--r--   1 jmackay  users        4964 May 19 15:35 extract.cpp\n"
      "-rw-r--r--   1 jmackay  users        6193 May 19 15:34 extract.h\n"
      "-rw-r--r--   1 jmackay  users        5659 May 19 15:35 extract.inl\n"
      "-rw-r--r--   1 jmackay  users       24556 Jun  2 21:24 extract.o\n"
      "-rw-r--r--   1 jmackay  users        1566 Jun  2 20:46 main.cpp\n"
      "-rw-r--r--   1 jmackay  users       39412 Jun  2 21:55 main.o\n"
      "-rw-r--r--   1 jmackay  users        7948 May 30 20:36 overlay.cpp\n"
      "-rw-r--r--   1 jmackay  users        5963 May 19 15:35 overlay.h\n"
      "-rw-r--r--   1 jmackay  users        2187 May 19 15:35 overlay.inl\n"
      "-rw-r--r--   1 jmackay  users       26472 Jun  2 21:25 overlay.o\n"
      "-rw-r--r--   1 jmackay  users        9163 Jun  2 20:50 pairfact.cpp\n"
      "-rw-r--r--   1 jmackay  users        2352 May 19 15:35 pairfact.h\n"
      "-rw-r--r--   1 jmackay  users        1953 May 19 15:35 pairfact.inl\n"
      "-rw-r--r--   1 jmackay  users       38528 Jun  2 21:25 pairfact.o\n"
      "-rw-r--r--   1 jmackay  users       16120 Jun  2 20:58 release.h";

   outStream << "#2" << endl;

   arrangeString(input1, "", outStream);				// 17 blank lines
   outStream << endl;
   arrangeString(input1, "1 2 2 1", outStream);		//  'r-' x 17
   outStream << endl;
																	// inode count, perms: 1 -rw...
   arrangeString(input1, "w 2 1 w 1 nextword", outStream); 
   outStream << endl;																
   arrangeString(input1, "w 2 next", outStream);	// inode count
   outStream << endl;
   } // test2()

//------------------------------------------------------------------------
// #3. Tests (parts of) all Extractors and Overlayers.
//------------------------------------------------------------------------
// all input range types  = { n, n.m, n.-, WORD n, WORD n.m, WORD n.- }
// all output range types = { n, n.m, NEXT, NEXT.m, NEXTWORD, NEXTWORD.m }  
//------------------------------------------------------------------------
void test3()
   {
   // Note: 3 lines in input1; last line does NOT end with a newline
   String input1 = 
      "-r--r--r--   1 jmackay  users          44 Jun  2 20:57 arrtest.val\n"
      "-rw-r-----   1 jmackay  users        5963 May 19 15:35 overlay.h\n"
      "-rwxr-----   1 jmackay  users        1953 May 19 15:35 pairfact.inl";

   outStream << "#3" << endl;

   // input range = n, output range = n
   // (rotates the first 4 chars 1 char to the right)
   arrangeString(input1, "1 2 2 3 3 4 4 1", outStream);
   outStream << endl;

   // input range = WORD n, output range = NEXT
   // (switches the first 2 words, eliminating any intervening whitespace)
   arrangeString(input1, "word 2 n W 1 NexT", outStream);
   outStream << endl;

   // input range = n.m, output range = NEXTWORD
   // ( [space][chars 6-15][space][chars 1-5] )
   arrangeString(input1, "6.10 NEXTW 1.5 nextWOR", outStream);
   outStream << endl;

   // input range = WORD n.m, output range = n.m
   // (first 5 chars of words 2-3 starting at pos 4; first char of first
   //  word at pos 1 (spaces at pos 2 and 3))
   arrangeString(input1, "w 2.2 4.5 w 1.1 1.1", outStream);
   outStream << endl;

   // input range = n.-, output range = NEXT.m
   // (chars 38-41 at pos 1; chars 47-48 at pos 5)
   arrangeString(input1, "38.- next.4 47.- next.2", outStream);
   outStream << endl;

   // input range = WORD n.-, output range = NEXTWORD.m
   // (first 15 chars of words 9- (filename) at pos 2; first 9 chars of words
   //  6- (month, day, hour) at pos 18 (spaces at pos 1 and 17))
   arrangeString(input1, "Word 9.- NextWord.15 Word 6.- NextWord.9", outStream);
   outStream << endl;
   } // test3()

//------------------------------------------------------------------------
// #4. Tests the parts of all Overlayers and Extractors that were missed
//     in the previous test.
//------------------------------------------------------------------------
void test4()
   {
   // 2 lines long; the last DOES end in a newline.
   String input1 = "This is a test string ...\nIt is only a test.\n";

   outStream << "#4" << endl;

   // tests CharExtractor::extract():
   arrangeString(input1, "80 1", outStream);      // 2 newlines
   outStream << endl;                             // + newline

   // tests CharToEndExtractor::extract():
   arrangeString(input1, "112.- 1", outStream);   // 2 newlines
   outStream << endl;                             // + newline

   // tests CharRangeExtractor::extract():
   arrangeString(input1, "140.35 1 24.20 next", outStream); // '..\n\n'
   outStream << endl;                                       // + newline
   } // test4()

//------------------------------------------------------------------------
// #5. Tests minimize().
//------------------------------------------------------------------------
void test5()
   {
   String input1 = "A shorter line,\n"
                   "and a (marginally) longer one that will, ideally, "
                   "increase the size of the buffer beyond its initial size.";
   Append  in  = Append::string(input1);

   String output;
   WriteTo out = WriteTo::string(output, WriteTo::OVERWRITE);

   Arrange arr("words 2.- 1");   // output all but the first word
   arr.connect(in, out);
   arr.minimize();

   outStream << "#5" << endl;
   outStream << output << endl;
   } // test5()

//------------------------------------------------------------------------
// #6. Tests error handling for the case where both an input range and
//     an output range are missing from an argument string.
//------------------------------------------------------------------------
void test6()
   {
   String input1 = "What`s in this string is unimportant - \n"
                   "it won`t be used for anything.\n";

   outStream << "#6" << endl;
   try
	   {
	   // should get an InvalidFilterArgumentException with a message
      // that both an input and output range is missing.
      arrangeString(input1, "1.5 2 Word", outStream);
      outStream << endl << "Exception wasn`t thrown - should have been!!!"
                << endl;
      }
   catch(InvalidFilterArgumentException &ifae)
      {
      if (ifae.getMessage().indexOf(MISSING_BOTH_RANGES_MSG) >= 0)
         outStream << "The correct exception (missing both ranges) "
                   << "has been caught." << endl;
      else
         outStream << "The incorrect exception was thrown !!!\n"
                   << "Its message is: " << ifae.getMessage() << endl;
      }
   } // test6()

//------------------------------------------------------------------------
// #7. Tests error handling for the case where there is an invalid input
//     range in the arguments string.
//------------------------------------------------------------------------
void test7()
   {
   String input1 = "What`s in this string is unimportant - \n"
                   "it won`t be used for anything.\n";

   outStream << "#7" << endl;
   try
	   {
	   // should get an InvalidFilterArgumentException with a message
      // that an input range is invalid.
      arrangeString(input1, "1.5 2 6 1 seven 6", outStream);
      outStream << endl << "Exception wasn`t thrown - should have been!!!"
                << endl;
      }
   catch(InvalidFilterArgumentException &ifae)
      {
      if (ifae.getMessage().indexOf(IS_INVALID_INPUT_RANGE_MSG) >= 0)
         outStream << "The correct exception (invalid input range) "
                   << "has been caught." << endl;
      else
         outStream << "The incorrect exception was thrown !!!\n"
                   << "Its message is: " << ifae.getMessage() << endl;
      }
   } // test7()

//------------------------------------------------------------------------
// #8. Tests error handling for the case where there is a missing output
//     range in the arguments string.
//------------------------------------------------------------------------
void test8()
   {
   String input1 = "What`s in this string is unimportant - \n"
                   "it won`t be used for anything.\n";

   outStream << "#8" << endl;
   try
	   {
	   // should get an InvalidFilterArgumentException with a message
      // that an output range is missing.
      arrangeString(input1, "11.- 1 1.10", outStream);
      outStream << endl << "Exception wasn`t thrown - should have been!!!"
                << endl;
      }
   catch(InvalidFilterArgumentException &ifae)
      {
      if (ifae.getMessage().indexOf(MISSING_OUTPUT_RANGE_MSG) >= 0)
         outStream << "The correct exception (missing output range) "
                   << "has been caught." << endl;
      else
         outStream << "The incorrect exception was thrown !!!\n"
                   << "Its message is: " << ifae.getMessage() << endl;
      }
   } // test8()

//------------------------------------------------------------------------
// #9. Tests error handling for the case where there is an invalid output
//     range in the arguments string.
//------------------------------------------------------------------------
void test9()
   {
   String input1 = "What`s in this string is unimportant - \n"
                   "it won`t be used for anything.\n";

   outStream << "#9" << endl;
   try
	   {
	   // should get an InvalidFilterArgumentException with a message
      // that an output range is invalid.
      arrangeString(input1, "11.- 1 1.10 .4", outStream);
      outStream << endl << "Exception wasn`t thrown - should have been!!!"
                << endl;
      }
   catch(InvalidFilterArgumentException &ifae)
      {
      if (ifae.getMessage().indexOf(IS_INVALID_OUTPUT_RANGE_MSG) >= 0)
         outStream << "The correct exception (invalid output range) "
                   << "has been caught." << endl;
      else
         outStream << "The incorrect exception was thrown !!!\n"
                   << "its message is: " << ifae.getMessage() << endl;
      }
   } // test9()

//------------------------------------------------------------------------
// #10. Test constructing an Arrange filter from an argument list.
//------------------------------------------------------------------------
void test10()
   {
   String input1 = "This is line the first\nthis is line the second\n"
                   "this is line the third\nthis is line the fourth";
   Append in = Append::string(input1);

   String output;
   WriteTo out = WriteTo::string(output, WriteTo::OVERWRITE);

   outStream << "#10" << endl;

   const int NUM_ARGS_1 = 5;
   char *argsList1[NUM_ARGS_1] =
      {
      "word", "4.-", "1", "w 3", "nextWORD"
      };
   Arrange arr1(NUM_ARGS_1, argsList1);
   arr1.connect(in, out);            // 'the xxxth line'  
   outStream << output << endl;

   // test the InvalidFilterArgumentException throw:
   const int NUM_ARGS_2 = 2;
   char *argsList2[NUM_ARGS_2] =
      {
      ".2", "1"
      };
   try
      {
      Arrange arr2(NUM_ARGS_2, argsList2);
      outStream << "Exception wasn`t thrown - should have been!!!" << endl;
      }
   catch(InvalidFilterArgumentException &ifae)
      {
      if (ifae.getMessage().indexOf(IS_INVALID_INPUT_RANGE_MSG) >= 0)
         outStream << "The correct exception (invalid input range) "
                   << "has been caught." << endl;
      else
         outStream << "The incorrect exception was thrown !!!\n"
                   << "Its message is: " << ifae.getMessage() << endl;
      }
   } // test10()

