//************************************************************************
// Declaration of the PairFactory class.
//------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//
//************************************************************************
#ifndef PAIRFACT_H_
#define PAIRFACT_H_

#include "extract.h"
#include "overlay.h"
#include "jstring.h"
#include "release.h"

//************************************************************************
// Class declarations.
//************************************************************************

//------------------------------------------------------------------------
// Class PairFactory.
//------------------------------------------------------------------------
class PairFactory
   {
   public:
      Extractor *createExtractor(String *args, String *errMsg);
      Overlayer *createOverlayer(String *args, String *errMsg);

      int countPairs(const String &args);
   private:
      static const String rangeSeparator;
      static const String toEndLength;

      static const String wordsTag;
      static const StringInt minWordsTagLength;

      static const String nextWordTag;
      static const StringInt minNextWordTagLength;

      static const int MAX_RANGE_DIGITS;

      boolean isValidRangePos(const String &pos) const;
      boolean isValidRangeLength(const String &len) const;
      StringInt stringToInt(const String &num) const;

   }; // class PairFactory

#ifdef ENABLE_INLINES
   #include "pairfact.inl"
#endif

#endif   // #ifndef PAIRFACT_H_

