//************************************************************************
// Declaration of the CharacterFilter filter class.
//------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//
//************************************************************************
#ifndef CHARFILT_H_
#define CHARFILT_H_

#include "filter.h"
#include "release.h"

//************************************************************************
// Class declarations.
//************************************************************************

//------------------------------------------------------------------------
// Abstract base class for filters that process their input a character at 
// a time.
//------------------------------------------------------------------------
class CharacterFilter: public SingleFilter
   {
   public:
      CharacterFilter();
      CharacterFilter(const CharacterFilter &f);

      virtual void processChar(char ch) = 0;

      void processLine(const String &line);
      void minimize();
   protected:
      void beforeInput();
      void afterInput();

      void outputChar(char ch);
   private:
      CharacterFilter &operator =(const CharacterFilter &f);
      void flushOutputBuffer();

      StringBuffer outputBuffer;

   }; // class CharacterFilter

#ifdef ENABLE_INLINES
   #include "charfilt.inl"
#endif

#endif   // #ifndef CHARFILT_H_

