//************************************************************************
// Declaration of the GenerateError filter class.
//------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//
//************************************************************************
#ifndef GENERROR_H_
#define GENERROR_H_

#include "filter.h"
#include "release.h"

//************************************************************************
// Class declarations.
//************************************************************************

//------------------------------------------------------------------------
// Filter that generates a FilterError from the specified message string
// at the specified point in processing.
//------------------------------------------------------------------------
class GenerateError: public SingleFilter
   {
   public:
      static GenerateError before(const String &errMsg);
      static GenerateError after(const String &errMsg);
      static GenerateError forLine(int lineNum, const String &errMsg);
      GenerateError(const GenerateError &f);

      void processLine(const String &line);

      DEFINE_FILTER_COPIER(GenerateError);
   protected:
      void beforeInput();
      void afterInput();
   private:
      GenerateError(int whenToGenerate, const String &errMsg);
      GenerateError &operator =(const GenerateError &f);

      static const int BEFORE;
      static const int AFTER;
 
      const String errorMsg;
      const int    errorLineNum;
      int          currLineNum;

   }; // class GenerateError

#ifdef ENABLE_INLINES
   #include "generror.inl"
#endif

#endif   // #ifndef GENERROR_H_

