//************************************************************************
// Automated test script for the String and StringBuffer classes.
//------------------------------------------------------------------------
// Copyright (C) 1997 by James MacKay
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The author can be contacted via e-mail at: jmackay@sentex.net
//
//************************************************************************

#include <fstream.h>
#include "jstring.h"
#include "release.h"

#define SEP '\n'
#define DLM '/'

// String tests
void test1();     void test2();     void test3();     void test4();
void test5();     void test6();     void test7();     void test8();
void test9();     void test10();    void test11();    void test12();
void test13();    void test14();    void test15();    void test16();
void test17();    void test18();    void test19();    void test20();
void test21();    void test22();    void test23();    void test24();
void test25();    void test26();    void test27();    void test28();
void test29();    void test30();    void test31();    void test32();
void test33();    void test34();    void test35();    void test36();
void test37();    void test38();    void test39();    void test40();
void test41();    void test42();    

// StringBuffer tests
void test1000();    void test1001();    void test1002();    void test1003();    
void test1004();    void test1005();    void test1006();    void test1007();    
void test1008();    void test1009();    void test1010();    void test1011();    
void test1012();

// WordIterator tests
void test2000();    void test2001();

#define outs cout
//ofstream outs("d:\\temp\\temp.out");

int main()
   {
   test1();
   test2();
   test3();
   test4();
   test5();
   test6();
   test7();
   test8();
   test9();
   test10();
   test11();
   test12();
   test13();
   test14();
   test15();
   test16();
   test17();
   test18();
   test19();
   test20();
   test21();
   test22();
   test23();
   test24();
   test25();
   test26();
   test27();
   test28();
   test29();
   test30();
   test31();
   test32();
   test33();
   test34();
   test35();
   test36();
   test37();
   test38();
   test39();
   test40();
   test41();
   test42();

   test1000();
   test1001();
   test1002();
   test1003();
   test1004();
   test1005();
   test1006();
   test1007();
   test1008();
   test1009();
   test1010();
   test1011();
   test1012();

   test2000();
   test2001();

   return 0;
   }

//************************************************************************
// Tests of class String.
//************************************************************************

//------------------------------------------------------------------------
// #1 String constructors (and the destructor).
//------------------------------------------------------------------------
void test1()
   {
   String str1;                           // ""
   String str2 = "a char array";          // "a char array"
   String str3 = str2;                    // "a char array"

   char temp[] = "Only part of this";
   String str4(temp, 5, 7);               // "part of"
   String str5(temp, 16, 0);              // ""
   String str6(temp, 14, 3);              // "his"

   StringBuffer buf;
   buf.append("buffer contents");
   String str7 = buf;                     // "buffer contents"

   String str8('*', 7);                   // "*******"
   String str9('\0', 15);                 // ""
   String str10('z', 0);                  // ""

   outs << "#1" << SEP 
        << DLM << str1 << DLM << SEP << DLM << str2 << DLM << SEP
        << DLM << str3 << DLM << SEP << DLM << str4 << DLM << SEP 
        << DLM << str5 << DLM << SEP << DLM << str6 << DLM << SEP 
        << DLM << str7 << DLM << SEP << DLM << str8 << DLM << SEP
        << DLM << str9 << DLM << SEP << DLM << str10 << DLM << SEP;

   }  // test1()

//------------------------------------------------------------------------
// #2 assignment and cast to (const char *) operators.
//------------------------------------------------------------------------
void test2()
   {
   String str1 = "the assigned value";
   String str2('Q', 20);
   String str3 = str1;

   str1 = str1;               // "the assigned value"
   str2 = str1;               // "the assigned value"
   str1 = str3;               // "the assigned value"

   const char *arr1 = str1;   // "the assigned value"

   outs << "#2" << SEP 
        << DLM << str1 << DLM << SEP << DLM << str2 << DLM << SEP
        << DLM << str3 << DLM << SEP << DLM << arr1 << DLM << SEP;
   }

//------------------------------------------------------------------------
// #3 copyValueOf().
//------------------------------------------------------------------------
void test3()
   {
   String str1 = String::copyValueOf("characters");
   String str2 = String::copyValueOf("array fragment", 10, 4); // "ment"

   outs << "#3" << SEP 
        << DLM << str1 << DLM << SEP << DLM << str2 << DLM << SEP;
   }

//------------------------------------------------------------------------
// #4 valueOf().
//------------------------------------------------------------------------
void test4()
   {
   char *arr = new char[15];
   strcpy(arr, "Actual string");
   String str1 = String::valueOf(arr);             // "Actual string"
   String str2 = String::valueOf(true);            // "true"
   String str3 = String::valueOf(false);           // "false"
   String str4 = String::valueOf('%');             // "%"
   String str5 = String::valueOf('\0');            // ""

   short int      si  = -7;
   unsigned short usi = 28;
   int            i   = 4000;
   unsigned       ui  = 0U;
   long           l   = -0L;
   unsigned long  ul  = 8675309L;
   String str6  = String::valueOf(si);             // "-7"
   String str7  = String::valueOf(usi);            // "28"
   String str8  = String::valueOf(i);              // "4000"
   String str9  = String::valueOf(ui);             // "0"
   String str10 = String::valueOf(l);              // "0"
   String str11 = String::valueOf(ul);             // "8675309"

   outs << "#4"  << SEP 
        << DLM << str1 << DLM << SEP << DLM << str2 << DLM << SEP
        << DLM << str3 << DLM << SEP << DLM << str4 << DLM << SEP 
        << DLM << str5 << DLM << SEP << DLM << str6 << DLM << SEP 
        << DLM << str7 << DLM << SEP << DLM << str8 << DLM << SEP
        << DLM << str9 << DLM << SEP << DLM << str10 << DLM << SEP 
        << DLM << str11 << DLM << SEP;

   }  // test4()

//------------------------------------------------------------------------
// #5 charAt() and length().
//------------------------------------------------------------------------
void test5()
   {
   String str1;
   String str2 = "A non-empty one.";   // length == 16

   outs << "#5" << SEP << str1.length() << SEP << str2.length() << SEP
        << DLM << str2.charAt(0) << DLM << SEP     // "/A/"
        << DLM << str2.charAt(1) << DLM << SEP     // "/ /"
        << DLM << str2.charAt(12) << DLM << SEP    // "/o/"
        << DLM << str2.charAt(13) << DLM << SEP    // "/n/"
        << DLM << str2.charAt(14) << DLM << SEP    // "/e/"
        << DLM << str2.charAt(15) << DLM << SEP    // "/./"
        << String::valueOf((str2.charAt(16) == '\0') ? true : false) << SEP;
                                                             // "true"
   }

//------------------------------------------------------------------------
// #6 compareTo()
//------------------------------------------------------------------------
void test6()
   {
   String str1 = "compare me!";
   String str2 = str1;
   String str3;
   String str4 = "Compare Me!";

   const int NUM_TESTS = 5;
   int result[NUM_TESTS];

   result[0] = str1.compareTo(str2);   // 0
   result[1] = str1.compareTo(str1);   // 0
   result[2] = str1.compareTo(str3);   // > 0
   result[3] = str3.compareTo(str1);   // < 0
   result[4] = str1.compareTo(str4);   // > 0 (lower case > upper)

   outs << "#6" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      if (result[i] > 0)
         outs << "greater than";
      else if (result[i] < 0)
         outs << "less than";
      else
         outs << "equal";
      outs << SEP;
      }
   }  // test6()

//------------------------------------------------------------------------
// #7 compareToIgnoreCase()
//------------------------------------------------------------------------
void test7()
   {
   String str1 = "compare me!";
   String str2 = str1;
   String str3;
   String str4 = "Compare Me!";

   const int NUM_TESTS = 5;
   int result[NUM_TESTS];

   result[0] = str1.compareToIgnoreCase(str2);   // 0
   result[1] = str1.compareToIgnoreCase(str1);   // 0
   result[2] = str1.compareToIgnoreCase(str3);   // > 0
   result[3] = str3.compareToIgnoreCase(str1);   // < 0
   result[4] = str1.compareToIgnoreCase(str4);   // 0

   outs << "#7" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      if (result[i] > 0)
         outs << "greater than";
      else if (result[i] < 0)
         outs << "less than";
      else
         outs << "equal";
      outs << SEP;
      }
   }  // test7()

//------------------------------------------------------------------------
// #8 equals() and equalsIgnoreCase().
//------------------------------------------------------------------------
void test8()
   {
   String str1 = "compare me!";
   String str2 = str1;
   String str3;
   String str4 = "Compare Me!";

   const int NUM_TESTS = 4;
   boolean result[NUM_TESTS];

   result[0] = str1.equals(str4);               // false
   result[1] = str1.equalsIgnoreCase(str4);     // true
   result[2] = str1.equals(str2);               // true
   result[3] = str3.equalsIgnoreCase(str1);     // false

   outs << "#8" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test8()

//------------------------------------------------------------------------
// #9 startsWith().
//------------------------------------------------------------------------
void test9()
   {
   String str1 = "The whole text.";
   String str2;

   const int NUM_TESTS = 10;
   boolean result[NUM_TESTS];

   result[0] = str1.startsWith(str1);           // true
   result[1] = str2.startsWith(str2);           // true
   result[2] = str1.startsWith(str2);           // true
   result[3] = str1.startsWith("the");          // false (case sensitive)
   result[4] = str1.startsWith("he");           // false

   result[5] = str1.startsWith(str1, 1);        // false
   result[6] = str1.startsWith(str2, 1);        // true
   result[7] = str1.startsWith("hole", 5);      // true
   result[8] = str1.startsWith("text.txt", 10); // false
   result[9] = str1.startsWith(" text.", 9);    // true

   outs << "#9" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test9()

//------------------------------------------------------------------------
// #10 endsWith().
//------------------------------------------------------------------------
void test10()
   {
   String str1 = "The whole text.";
   String str2;
   String str3 = "The whole text";

   const int NUM_TESTS = 8;
   boolean result[NUM_TESTS];

   result[0] = str1.endsWith(str1);          // true
   result[1] = str2.endsWith(str2);          // true
   result[2] = str1.endsWith(str2);          // true
   result[3] = str1.endsWith("XT.");         // false (case sensitive)
   result[4] = str1.endsWith("ext");         // false
   result[5] = str1.endsWith("ext..");       // false
   result[6] = str2.endsWith(str1);          // false
   result[7] = str3.endsWith(str1);          // false

   outs << "#10" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test10()

//------------------------------------------------------------------------
// #11 regionMatches().
//------------------------------------------------------------------------
void test11()
   {
   String str1 = "regions Regions REGIONS";
   String str2;
   String str3 = "anything";

   const int NUM_TESTS = 7;
   boolean result[NUM_TESTS];

   result[0] = str1.regionMatches(0, str2, 0, 0);                 // true
   result[1] = str1.regionMatches(22, str2, 0, 0);                // true
   result[2] = str1.regionMatches(5, str3, 3, 0);                 // true
   result[3] = str1.regionMatches(0, str2, 0, 1);                 // false
   result[4] = str1.regionMatches(22, "regionS", 6, 1);           // true
   result[5] = str1.regionMatches(22, "regionS", 6, 5);           // true
   result[6] = str1.regionMatches(8, "two regions only", 4, 8);   // false

   outs << "#11" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test11()

//------------------------------------------------------------------------
// #12 regionMatchesIgnoreCase()
//------------------------------------------------------------------------
void test12()
   {
   String str1 = "regions Regions REGIONS";
   String str2;
   String str3 = "anything";

   const int NUM_TESTS = 7;
   boolean result[NUM_TESTS];

   result[0] = str1.regionMatchesIgnoreCase(0, str2, 0, 0);         // true
   result[1] = str1.regionMatchesIgnoreCase(22, str2, 0, 0);        // true
   result[2] = str1.regionMatchesIgnoreCase(5, str3, 3, 0);         // true
   result[3] = str1.regionMatchesIgnoreCase(0, str2, 0, 1);         // false
   result[4] = str1.regionMatchesIgnoreCase(22, "regionS", 6, 1);   // true
   result[5] = str1.regionMatchesIgnoreCase(22, "regionS", 6, 5);   // true
   result[6] = str1.regionMatchesIgnoreCase(8, "two regions only", 4, 8);
                                                                    // true

   outs << "#12" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test12()

//------------------------------------------------------------------------
// #13 isAbbreviationOf().
//------------------------------------------------------------------------
void test13()
   {
   String str1 = "establishment";
   String str2;
   String str3 = "est";

   const int NUM_TESTS = 9;
   boolean result[NUM_TESTS];

   result[0] = str3.isAbbreviationOf(str1, 0);           // true
   result[1] = str3.isAbbreviationOf(str1, 3);           // true
   result[2] = str3.isAbbreviationOf(str1, 4);           // false
   result[3] = str1.isAbbreviationOf(str3, 0);           // false
   result[4] = str1.isAbbreviationOf(str3, 3);           // false
   result[5] = str2.isAbbreviationOf(str1, 0);           // true
   result[6] = str2.isAbbreviationOf(str1, 1);           // false
   result[7] = str3.isAbbreviationOf(str2, 0);           // false
   result[8] = String("Est").isAbbreviationOf(str1, 0);  // false

   outs << "#13" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test13()

//------------------------------------------------------------------------
// #14 indexOf().
//------------------------------------------------------------------------
void test14()
   {
   String str1 = "The string to Search";  // length == 20
   String str2;
   const int NUM_TESTS = 18;
   StringInt result[NUM_TESTS];

   result[0] = str1.indexOf(String::terminator());          // -1
   result[1] = str1.indexOf(String::terminator(), 20);      // -1
   result[2] = str1.indexOf('S');                           // 14
   result[3] = str1.indexOf(' ', 14);                       // -1
   result[4] = str1.indexOf(' ', 13);                       // 13
   result[5] = str1.indexOf('b', 5);                        // -1
   result[6] = str2.indexOf(String::terminator(), 0);       // -1
   result[7] = str2.indexOf('d');                           // -1

   result[8] = str2.indexOf(str2);                          // -1
   result[9] = str2.indexOf(str1, 0);                       // -1
   result[10] = str1.indexOf(str2);                         // 0
   result[11] = str1.indexOf(str2, 7);                      // 7
   result[12] = str1.indexOf("str");                        // 4
   result[13] = str1.indexOf("str", 5);                     // -1
   result[14] = str1.indexOf("str", 4);                     // 4
   result[15] = str1.indexOf(str2, 20);                     // -1
   result[16] = str1.indexOf("try", 3);                     // -1

   result[17] = str1.indexOf('h', 20);                      // -1

   outs << "#14" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test14()

//------------------------------------------------------------------------
// #15 lastIndexOf().
//------------------------------------------------------------------------
void test15()
   {
   String str1 = "Can you find it in this??";   // length == 25
   String str2;
   const int NUM_TESTS = 22;
   StringInt result[NUM_TESTS];

   result[0] = str1.lastIndexOf(String::terminator());        // -1
   result[1] = str1.lastIndexOf(String::terminator(), 25);    // -1
   result[2] = str1.lastIndexOf(String::terminator(), 12);    // -1
   result[3] = str1.lastIndexOf('o');                         // 5
   result[4] = str1.lastIndexOf('n');                         // 17
   result[5] = str1.lastIndexOf('n', 17);                     // 17
   result[6] = str1.lastIndexOf('n', 16);                     // 10
   result[7] = str1.lastIndexOf('C');                         // 0
   result[8] = str1.lastIndexOf('c');                         // -1
   result[9] = str2.lastIndexOf(String::terminator(), 0);     // -1
   result[10] = str2.lastIndexOf('y');                        // -1

   result[11] = str2.lastIndexOf(str2);                       // -1
   result[12] = str2.lastIndexOf(str1);                       // -1
   result[13] = str2.lastIndexOf(str1, 0);                    // -1
   result[14] = str2.lastIndexOf(str2, 0);                    // -1
   result[15] = str1.lastIndexOf("in");                       // 16
   result[16] = str1.lastIndexOf("in", 16);                   // 16
   result[17] = str1.lastIndexOf("in", 15);                   // 9
   result[18] = str1.lastIndexOf("in", 8);                    // -1
   result[19] = str1.lastIndexOf(str2);                       // 24
   result[20] = str1.lastIndexOf(str2, 13);                   // 13
   result[21] = str1.lastIndexOf(str2, 25);                   // 24

   outs << "#15" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test15()

//------------------------------------------------------------------------
// #16 indexOfCharIn().
//------------------------------------------------------------------------
void test16()
   {
   String str1 = "A string containing several letters";  // length == 35
   String str2;
   const int NUM_TESTS = 11;
   StringInt result[NUM_TESTS];

   result[0] = str2.indexOfCharIn("");                   // -1
   result[1] = str1.indexOfCharIn("");                   // -1
   result[2] = str1.indexOfCharIn("", 35);               // -1
   result[3] = str1.indexOfCharIn("aeiou");              // 5
   result[4] = str1.indexOfCharIn("aeiou", 5);           // 5
   result[5] = str1.indexOfCharIn("aeiou", 6);           // 10
   result[6] = str1.indexOfCharIn("xyz");                // -1
   result[7] = str2.indexOfCharIn("xyz", 0);             // -1
   result[8] = str2.indexOfCharIn("xyz");                // -1
   result[9] = str1.indexOfCharIn("A");                  // 0
   result[10] = str1.indexOfCharIn("A", 1);              // -1

   outs << "#16" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test16()

//------------------------------------------------------------------------
// #17 indexOfCharNotIn()
//------------------------------------------------------------------------
void test17()
   {
   String str1 = "A string containing several letters";  // length == 35
   String str2;
   const int NUM_TESTS = 17;
   StringInt result[NUM_TESTS];

   result[0] = str2.indexOfCharNotIn("");                   // -1
   result[1] = str1.indexOfCharNotIn("");                   // 0
   result[2] = str1.indexOfCharNotIn("", 35);               // -1
   result[3] = str1.indexOfCharNotIn("", 13);               // 13
   result[4] = str1.indexOfCharNotIn("aeiou");              // 0
   result[5] = str1.indexOfCharNotIn("Against her");        // 9
   result[6] = str1.indexOfCharNotIn("Against her", 9);     // 9
   result[7] = str1.indexOfCharNotIn("Against her", 10);    // 10
   result[8] = str1.indexOfCharNotIn("Aacegilnorstv ");     // -1
   result[9] = str2.indexOfCharNotIn("xyz", 0);             // -1
   result[10] = str2.indexOfCharNotIn("xyz");               // -1
   result[11] = str1.indexOfCharNotIn("A");                 // 1

   result[12] = str2.indexOfCharNotIn(String::terminator());      // -1
   result[13] = str1.indexOfCharNotIn(String::terminator(), 5);   // 5
   result[14] = str2.indexOfCharNotIn('e', 0);                    // -1
   result[15] = str1.indexOfCharNotIn('o', 10);                   // 11
   result[16] = str1.indexOfCharNotIn('o', 9);                    // 9

   outs << "#17" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test17()

//------------------------------------------------------------------------
// #18 lastIndexOfCharIn().
//------------------------------------------------------------------------
void test18()
   {
   String str1 = "A string containing several letters";  // length == 35
   String str2;
   const int NUM_TESTS = 8;
   StringInt result[NUM_TESTS];

   result[0] = str2.lastIndexOfCharIn("");                  // -1
   result[1] = str2.lastIndexOfCharIn("aeiou", 0);          // -1
   result[2] = str1.lastIndexOfCharIn("blink!");            // 28
   result[3] = str1.lastIndexOfCharIn("blink!", 28);        // 28
   result[4] = str1.lastIndexOfCharIn("blink!", 27);        // 26
   result[5] = str1.lastIndexOfCharIn("xyz");               // -1
   result[6] = str1.lastIndexOfCharIn("xyz", 35);           // -1
   result[7] = str1.lastIndexOfCharIn("ABCD", 0);           // 0

   outs << "#18" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test18()

//------------------------------------------------------------------------
// #19 lastIndexOfCharNotIn()
//------------------------------------------------------------------------
void test19()
   {
   String str1 = "A string containing several letters";  // length == 35
   String str2;
   const int NUM_TESTS = 16;
   StringInt result[NUM_TESTS];

   result[0] = str2.lastIndexOfCharNotIn(str2);                      // -1
   result[1] = str2.lastIndexOfCharNotIn("xyz", 0);                  // -1
   result[2] = str2.lastIndexOfCharNotIn(String::terminator(), 0);   // -1
   result[3] = str2.lastIndexOfCharNotIn(String::terminator());      // -1
   result[4] = str1.lastIndexOfCharNotIn("rest");                    // 28
   result[5] = str1.lastIndexOfCharNotIn("reveals", 26);             // 19
   result[6] = str1.lastIndexOfCharNotIn("again", 19);               // 19
   result[7] = str1.lastIndexOfCharNotIn("again", 18);               // 12
   result[8] = str1.lastIndexOfCharNotIn(str2, 35);                  // 34
   result[9] = str1.lastIndexOfCharNotIn(str2, 8);                   // 8
   result[10] = str1.lastIndexOfCharNotIn(str2);                     // 34
   result[11] = str1.lastIndexOfCharNotIn(String::terminator(), 35); // 34
   result[12] = str1.lastIndexOfCharNotIn('s');                      // 33
   result[13] = str1.lastIndexOfCharNotIn('t', 31);                  // 29
   result[14] = str1.lastIndexOfCharNotIn('A', 0);                   // -1
   result[15] = str1.lastIndexOfCharNotIn(String::terminator());     // 34

   outs << "#19" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test19()

//------------------------------------------------------------------------
// #20 whitespaceIndex().
//------------------------------------------------------------------------
void test20()
   {
   String str1 = "White\tXmas\nand a\n\tHappy \tEaster \n!!"; // len == 35
   String str2;
   const int NUM_TESTS = 8;
   StringInt result[NUM_TESTS];

   result[0] = str2.whitespaceIndex();                // -1
   result[1] = str2.whitespaceIndex(0);               // -1
   result[2] = str1.whitespaceIndex();                // 5
   result[3] = str1.whitespaceIndex(5);               // 5
   result[4] = str1.whitespaceIndex(6);               // 10
   result[5] = str1.whitespaceIndex(11);              // 14
   result[6] = str1.whitespaceIndex(35);              // -1
   result[7] = str1.whitespaceIndex(34);              // -1

   outs << "#20" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test20()

//------------------------------------------------------------------------
// #21 nonWhitespaceIndex()
//------------------------------------------------------------------------
void test21()
   {
   String str1 = "White\tXmas\nand a\n\tHappy \tEaster \n!!"; // len == 35
   String str2;
   const int NUM_TESTS = 11;
   StringInt result[NUM_TESTS];

   result[0] = str2.nonWhitespaceIndex();          // -1
   result[1] = str2.nonWhitespaceIndex(0);         // -1
   result[2] = str1.nonWhitespaceIndex();          // 0
   result[3] = str1.nonWhitespaceIndex(4);         // 4
   result[4] = str1.nonWhitespaceIndex(5);         // 6
   result[5] = str1.nonWhitespaceIndex(6);         // 6
   result[6] = str1.nonWhitespaceIndex(10);        // 11
   result[7] = str1.nonWhitespaceIndex(14);        // 15
   result[8] = str1.nonWhitespaceIndex(16);        // 18
   result[9] = str1.nonWhitespaceIndex(35);        // -1
   result[10] = str1.nonWhitespaceIndex(34);       // 34

   outs << "#21" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test21()

//------------------------------------------------------------------------
// #22 lastWhitespaceIndex().
//------------------------------------------------------------------------
void test22()
   {
   String str1 = "White\tXmas\nand a\n\tHappy \tEaster \n!!"; // len == 35
   String str2;
   const int NUM_TESTS = 8;
   StringInt result[NUM_TESTS];

   result[0] = str2.lastWhitespaceIndex();      // -1
   result[1] = str2.lastWhitespaceIndex(0);     // -1
   result[2] = str1.lastWhitespaceIndex();      // 32
   result[3] = str1.lastWhitespaceIndex(35);    // 32
   result[4] = str1.lastWhitespaceIndex(31);    // 31
   result[5] = str1.lastWhitespaceIndex(30);    // 24
   result[6] = str1.lastWhitespaceIndex(5);     // 5
   result[7] = str1.lastWhitespaceIndex(4);     // -1

   outs << "#22" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test22()

//------------------------------------------------------------------------
// #23 lastNonWhitespaceIndex().
//------------------------------------------------------------------------
void test23()
   {
   String str1 = "White\tXmas\nand a\n\tHappy \tEaster \n!!"; // len == 35
   String str2;
   String str3 = "    and some non-spaces.";
   const int NUM_TESTS = 13;
   StringInt result[NUM_TESTS];

   result[0] = str2.lastNonWhitespaceIndex();      // -1
   result[1] = str2.lastNonWhitespaceIndex(0);     // -1
   result[2] = str1.lastNonWhitespaceIndex();      // 34
   result[3] = str1.lastNonWhitespaceIndex(35);    // 34
   result[4] = str1.lastNonWhitespaceIndex(0);     // 0
   result[5] = str3.lastNonWhitespaceIndex(4);     // 4
   result[6] = str3.lastNonWhitespaceIndex(3);     // -1
   result[7] = str1.lastNonWhitespaceIndex(33);    // 33
   result[8] = str1.lastNonWhitespaceIndex(32);    // 30
   result[9] = str1.lastNonWhitespaceIndex(25);    // 25
   result[10] = str1.lastNonWhitespaceIndex(24);   // 22
   result[11] = str1.lastNonWhitespaceIndex(15);   // 15
   result[12] = str1.lastNonWhitespaceIndex(14);   // 13

   outs << "#23" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test23()

//------------------------------------------------------------------------
// #24 digitIndex().
//------------------------------------------------------------------------
void test24()
   {
   String str1 = "(22 / 7) * 3.1415926536";  // length == 23
   String str2;
   const int NUM_TESTS = 8;
   StringInt result[NUM_TESTS];

   result[0] = str2.digitIndex();         // -1
   result[1] = str2.digitIndex(0);        // -1
   result[2] = str1.digitIndex();         // 1
   result[3] = str1.digitIndex(23);       // -1
   result[4] = str1.digitIndex(22);       // 22
   result[5] = str1.digitIndex(12);       // 13
   result[6] = str1.digitIndex(11);       // 11
   result[7] = str1.digitIndex(3);        // 6

   outs << "#24" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test24()

//------------------------------------------------------------------------
// #25 nonDigitIndex().
//------------------------------------------------------------------------
void test25()
   {
   String str1 = "(22 / 7) * 3.1415926536";  // length == 23
   String str2;
   const int NUM_TESTS = 9;
   StringInt result[NUM_TESTS];

   result[0] = str2.nonDigitIndex();         // -1
   result[1] = str2.nonDigitIndex(0);        // -1
   result[2] = str1.nonDigitIndex();         // 0
   result[3] = str1.nonDigitIndex(2);        // 3
   result[4] = str1.nonDigitIndex(1);        // 3
   result[5] = str1.nonDigitIndex(23);       // -1
   result[6] = str1.nonDigitIndex(22);       // -1
   result[7] = str1.nonDigitIndex(13);       // -1
   result[8] = str1.nonDigitIndex(12);       // 12

   outs << "#25" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test25()

//------------------------------------------------------------------------
// #26 concat() and operator +().
//------------------------------------------------------------------------
void test26()
   {
   String str1 = "First part:";
   String str2 = ":Second part.";
   String str3;
   const int NUM_TESTS = 8;
   String result[NUM_TESTS];

   result[0] = str3.concat(str3);      // ""
   result[1] = str3 + str3;            // ""
   result[2] = str1.concat(str3);      // "First part:"
   result[3] = str1 + str3;            // "First part:"
   result[4] = str1.concat(str2);      // "First part::Second part."
   result[5] = str1 + str2;            // "First part::Second part."
   result[6] = str3.concat(str2);      // ":Second part."
   result[7] = str3 + str2;            // ":Second part."

   outs << "#26" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test26()

//------------------------------------------------------------------------
// #27 copies().
//------------------------------------------------------------------------
void test27()
   {
   String str1 = "CopyThis";
   String str2;
   const int NUM_TESTS = 6;
   String result[NUM_TESTS];

   result[0] = str2.copies(0);         // ""
   result[1] = str2.copies(1);         // ""
   result[2] = str2.copies(10);        // ""
   result[3] = str1.copies(0);         // ""
   result[4] = str1.copies(1);         // "CopyThis"
   result[5] = str1.copies(3);         // "CopyThisCopyThisCopyThis"

   outs << "#27" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test27()

//------------------------------------------------------------------------
// #28 replace().
//------------------------------------------------------------------------
void test28()
   {
   String str1 = "0 1 2 3 4.";
   String str2;
   const int NUM_TESTS = 7;
   String result[NUM_TESTS];

   result[0] = str2.replace(String::terminator(), '!');  // ""
   result[1] = str2.replace(' ', '-');                   // ""
   result[2] = str1.replace(' ', '+');                   // "0+1+2+3+4."
   result[3] = str1.replace('0', '=');                   // "= 1 2 3 4."
   result[4] = str1.replace('.', 's');                   // "0 1 2 3 4s"
   result[5] = str1.replace('2', '0');                   // "0 1 0 3 4."
   result[6] = str1.replace('5', '%');                   // "0 1 2 3 4."

   outs << "#28" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test28()

//------------------------------------------------------------------------
// #29 translate().
//------------------------------------------------------------------------
void test29()
   {
   String str1 = "0 1 2 3 4.";
   String str2;
   const int NUM_TESTS = 7;
   String result[NUM_TESTS];

   result[0] = str2.translate("abcd", "1234");           // ""
   result[1] = str2.translate("", "");                   // ""
   result[2] = str1.translate("01234. ", "abcde -");     // "a-b-c-d-e "
   result[3] = str1.translate("01234", "43210");         // "4 3 2 1 0."
   result[4] = str1.translate("01210", "abcde");         // "a b c 3 4."
   result[5] = str1.translate("", "");                   // "0 1 2 3 4."
   result[6] = str1.translate(" ", ".");                 // "0.1.2.3.4."

   outs << "#29" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test29()

//------------------------------------------------------------------------
// #30 substring().
//------------------------------------------------------------------------
void test30()
   {
   String str1 = "We Care A Lot";   // length == 13
   String str2;
   const int NUM_TESTS = 14;
   String result[NUM_TESTS];

   result[0] = str2.substring(0);               // ""
   result[1] = str2.substring(0, 0);            // ""
   result[2] = str1.substring(0);               // "We Care A Lot"
   result[3] = str1.substring(0, 12);           // "We Care A Lot"
   result[4] = str1.substring(0, 13);           // "We Care A Lot"
   result[5] = str1.substring(13, 13);          // ""
   result[6] = str1.substring(12, 12);          // "t"
   result[7] = str1.substring(8);               // "A Lot"
   result[8] = str1.substring(3, 6);            // "Care"
   result[9] = str1.substring(3, 7);            // "Care "
   result[10] = str1.substring(13);             // ""
   result[11] = str1.substring(12);             // "t"
   result[12] = str1.substring(12, 11);         // ""
   result[13] = str1.substring(12, 0);          // ""

   outs << "#30" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test30()

//------------------------------------------------------------------------
// #31 left().
//------------------------------------------------------------------------
void test31()
   {
   String str1 = "Length == 12";
   String str2;
   const int NUM_TESTS = 7;
   String result[NUM_TESTS];

   result[0] = str2.left(0);        // ""
   result[1] = str2.left(5);        // "     " (5 spaces)
   result[2] = str1.left(0);        // ""
   result[3] = str1.left(3);        // "Len"
   result[4] = str1.left(14);       // "Length == 12  "
   result[5] = str1.left(12);       // "Length == 12"
   result[6] = str1.left(1);        // "L"

   outs << "#31" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test31()

//------------------------------------------------------------------------
// #32 right().
//------------------------------------------------------------------------
void test32()
   {
   String str1 = "Length == 12";
   String str2;
   const int NUM_TESTS = 7;
   String result[NUM_TESTS];

   result[0] = str2.right(0);        // ""
   result[1] = str2.right(5);        // "     " (5 spaces)
   result[2] = str1.right(0);        // ""
   result[3] = str1.right(3);        // " 12"
   result[4] = str1.right(14);       // "  Length == 12"
   result[5] = str1.right(12);       // "Length == 12"
   result[6] = str1.right(1);        // "2"

   outs << "#32" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test32()

//------------------------------------------------------------------------
// #33 centre() and center().
//------------------------------------------------------------------------
void test33()
   {
   String str1 = "Length == 12";
   String str2;
   String str3 = "Length 9.";
   const int NUM_TESTS = 20;
   String result[NUM_TESTS];

   result[0] = str2.center(0);         // ""
   result[1] = str2.centre(6);         // "      " (6 spaces)
   result[2] = str2.center(5);         // "     "  (5 spaces)
   result[3] = str1.centre(12);        // "Length == 12"
   result[4] = str1.centre(0);         // ""
   result[5] = str1.centre(1);         // "h"
   result[6] = str1.center(2);         // "h "
   result[7] = str1.centre(3);         // "th "
   result[8] = str1.center(11);        // "Length == 1"
   result[9] = str1.centre(13);        // "Length == 12 "
   result[10] = str1.center(14);       // " Length == 12 "
   result[11] = str1.centre(17);       // "  Length == 12   "
   result[12] = str3.centre(1);        // "t"
   result[13] = str3.centre(2);        // "gt"
   result[14] = str3.centre(0);        // ""
   result[15] = str3.centre(8);        // "Length 9"
   result[16] = str3.centre(9);        // "Length 9."
   result[17] = str3.centre(10);       // "Length 9. "
   result[18] = str3.centre(11);       // " Length 9. "
   result[19] = str3.centre(16);       // "   Length 9.    " (4 tr. spaces)

   outs << "#33" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test33()

//------------------------------------------------------------------------
// #34 strip() and trim().
//------------------------------------------------------------------------
void test34()
   {
   String str1 = "  \t some other \n ";
   String str2;
   const int NUM_TESTS = 7;
   String result[NUM_TESTS];

   result[0] = str2.strip();           // ""
   result[1] = str2.trim();            // ""
   result[2] = str2.strip('$');        // ""
   result[3] = str1.strip();           // "some other"
   result[4] = str1.trim();            // "some other"
   result[5] = str1.strip(' ');        // "\t some other \n"
   result[6] = str1.strip('s');        // "  \t some other \n "

   outs << "#34" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i].replace(' ', '+')
           << DLM << SEP;   // "/[string]/"
      }
   }  // test34()

//------------------------------------------------------------------------
// #35 stripLeading() and stripTrailing().
//------------------------------------------------------------------------
void test35()
   {
   String str1 = "  \t some other \n ";
   String str2;
   const int NUM_TESTS = 10;
   String result[NUM_TESTS];

   result[0] = str2.stripLeading();          // ""
   result[1] = str2.stripTrailing();         // ""
   result[2] = str2.stripLeading('$');       // ""
   result[3] = str2.stripTrailing('$');      // ""
   result[4] = str1.stripLeading();          // "some other \n "
   result[5] = str1.stripTrailing();         // "  \t some other"
   result[6] = str1.stripLeading(' ');       // "\t some other \n "
   result[7] = str1.stripTrailing(' ');      // "  \t some other \n"
   result[8] = str1.stripLeading('s');       // "  \t some other \n "
   result[9] = str1.stripTrailing('s');      // "  \t some other \n "

   outs << "#35" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i].replace(' ', '+')
           << DLM << SEP;   // "/[string]/"
      }
   }  // test35()

//------------------------------------------------------------------------
// #36 toLowerCase(),toUpperCase() and toString().
//------------------------------------------------------------------------
void test36()
   {
   String str1 = "A mIXed caSE SENTenCe.";
   String str2;
   String str3 = "ALL UPPER CASE!!";
   String str4 = "and all lower";
   const int NUM_TESTS = 12;
   String result[NUM_TESTS];

   result[0] = str1.toUpperCase();  // "A MIXED CASE SENTENCE."
   result[1] = str1.toLowerCase();  // "a mixed case sentence."
   result[2] = str2.toUpperCase();  // ""
   result[3] = str2.toLowerCase();  // ""
   result[4] = str3.toUpperCase();  // "ALL UPPER CASE!!"
   result[5] = str3.toLowerCase();  // "all upper case!!"
   result[6] = str4.toUpperCase();  // "AND ALL LOWER"
   result[7] = str4.toLowerCase();  // "and all lower"
   result[8] = str1.toString();     // "A mIXed caSE SENTenCe."
   result[9] = str2.toString();     // ""
   result[10] = str3.toString();    // "ALL UPPER CASE!!"
   result[11] = str4.toString();    // "and all lower"

   outs << "#36" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test36()

//------------------------------------------------------------------------
// #37 wordCount().
//------------------------------------------------------------------------
void test37()
   {
   String str1 = "A several-worded sentence, this.";  // length == 32
   String str2;
   String str3 = "  Starts with whitespace,\nand ends with it,\ttoo.  ";
                                                      // length == 50
   const int NUM_TESTS = 13;
   StringInt result[NUM_TESTS];

   result[0] = str1.wordCount();       // 4
   result[1] = str2.wordCount();       // 0
   result[2] = str3.wordCount();       // 8
   result[3] = str1.wordCount(2);      // 3
   result[4] = str1.wordCount(3);      // 3
   result[5] = str1.wordCount(15);     // 3
   result[6] = str1.wordCount(16);     // 2
   result[7] = str1.wordCount(32);     // 0
   result[8] = str1.wordCount(31);     // 1
   result[9] = str2.wordCount(0);      // 0
   result[10] = str3.wordCount(47);    // 1
   result[11] = str3.wordCount(48);    // 0
   result[12] = str3.wordCount(8);     // 7

   outs << "#37" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << result[i] << SEP;
      }
   }  // test37()

//------------------------------------------------------------------------
// #38 wordLength().
//------------------------------------------------------------------------
void test38()
   {
   String str1 = "A several-worded sentence, this.";  // length == 32
   String str2;
   const int NUM_TESTS = 12;
   StringInt result[NUM_TESTS];

   result[0] = str2.wordLength(0);           // 0
   result[1] = str2.wordLength(0, 0);        // 0
   result[2] = str1.wordLength(0);           // 1
   result[3] = str1.wordLength(3);           // 5
   result[4] = str1.wordLength(4);           // 0
   result[5] = str1.wordLength(0, 1);        // 14
   result[6] = str1.wordLength(1, 9);        // 9
   result[7] = str1.wordLength(0, 32);       // 0
   result[8] = str1.wordLength(0, 31);       // 1
   result[9] = str1.wordLength(0, 30);       // 2
   result[10] = str1.wordLength(0, 26);      // 5
   result[11] = str1.wordLength(2, 15);      // 5

   outs << "#38" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << result[i] << SEP;
      }
   }  // test38()

//------------------------------------------------------------------------
// #39 indexOfWord().
//------------------------------------------------------------------------
void test39()
   {
   String str1 = "A several-worded sentence, this.";  // length == 32
   String str2;
   String str3 = "  Starts with whitespace,\nand ends with it,\ttoo.  ";
                                                      // length == 50
   const int NUM_TESTS = 14;
   StringInt result[NUM_TESTS];

   result[0] = str2.indexOfWord(0);          // -1
   result[1] = str2.indexOfWord(0, 0);       // -1
   result[2] = str2.indexOfWord(5);          // -1
   result[3] = str1.indexOfWord(0);          // 0
   result[4] = str1.indexOfWord(0, 1);       // 2
   result[5] = str1.indexOfWord(0, 32);      // -1
   result[6] = str1.indexOfWord(0, 31);      // 31
   result[7] = str1.indexOfWord(2, 15);      // 27
   result[8] = str1.indexOfWord(1, 16);      // 27
   result[9] = str1.indexOfWord(2, 16);      // -1
   result[10] = str3.indexOfWord(0);         // 2
   result[11] = str3.indexOfWord(0, 48);     // -1
   result[12] = str3.indexOfWord(3, 48);     // -1
   result[13] = str3.indexOfWord(2, 24);     // 30 (ends)

   outs << "#39" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << result[i] << SEP;
      }
   }  // test39()

//------------------------------------------------------------------------
// #40 word().
//------------------------------------------------------------------------
void test40()
   {
   String str1 = "A several-worded sentence, this.";  // length == 32
   String str2;
   String str3 = "  Starts with whitespace,\nand ends with it,\ttoo.  ";
   const int NUM_TESTS = 16;
   String result[NUM_TESTS];

   result[0] = str2.word(0);        // ""
   result[1] = str2.word(1);        // ""
   result[2] = str2.word(0, 0);     // ""
   result[3] = str1.word(0);        // "A"
   result[4] = str3.word(0);        // "Starts"
   result[5] = str1.word(0, 1);     // "several-worded"
   result[6] = str1.word(0, 2);     // "several-worded"
   result[7] = str1.word(0, 3);     // "everal-worded"
   result[8] = str1.word(3);        // "this."
   result[9] = str1.word(4);        // ""
   result[10] = str3.word(4);       // "ends"
   result[11] = str3.word(7);       // "too."
   result[12] = str3.word(8);       // ""
   result[13] = str3.word(15);      // ""
   result[14] = str3.word(2, 7);    // "whitespace,"
   result[15] = str3.word(2, 8);    // "and"

   outs << "#40" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test40()

//------------------------------------------------------------------------
// #41 subword().
//------------------------------------------------------------------------
void test41()
   {
   String str1 = "A several-worded sentence, this.";  // length == 32
   String str2;
   String str3 = "  Starts with whitespace,\nand ends with it,\ttoo.  ";
   const int NUM_TESTS = 16;
   String result[NUM_TESTS];

   result[0] = str2.subword(0);           // ""
   result[1] = str2.subword(0, 0);        // ""
   result[2] = str2.subword(0, 0, 0);     // ""
   result[3] = str1.subword(2);           // "sentence, this."
   result[4] = str1.subword(0, 0);        // "A"
   result[5] = str1.subword(0, 1, 1);     // "several-worded sentence,"
   result[6] = str3.subword(6);           // "it,\ttoo."
   result[7] = str3.subword(0, 2, 4);     // "arts with whitespace,"
   result[8] = str3.subword(2, 3, 6);     // "whitespace,\nand"
   result[9] = str3.subword(1, 2, 6);     // "with whitespace,"
   result[10] = str3.subword(3, 6);       // "and ends with it,"
   result[11] = str3.subword(3, 7);       // "and ends with it,\ttoo."
   result[12] = str3.subword(8);          // ""
   result[13] = str3.subword(7);          // "too."
   result[14] = str3.subword(12);         // ""
   result[15] = str3.subword(7, 10);      // "too."

   outs << "#41" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test41()

//------------------------------------------------------------------------
// #42 getChars() and toCharArray().
//------------------------------------------------------------------------
void test42()
   {
   String str1 = "ThisIsThe Source String?";    // length == 24
   String str2;
   char *dest1 = str1.toCharArray();
   char *dest2 = str2.toCharArray();

   outs << "#42" << SEP;
   outs << DLM << dest1 << DLM << SEP       // "ThisIsThe Source String?"
        << DLM << dest2 << DLM << SEP;      // ""

   str2.getChars(0, 0, dest1, 0);      // "ThisIsThe Source String?"
   outs << DLM << dest1 << DLM << SEP;
   str2.getChars(0, 0, dest2, 0);      // ""
   outs << DLM << dest2 << DLM << SEP;
   str2.getChars(0, 0, dest1, 24);     // "ThisIsThe Source String?"
   outs << DLM << dest1 << DLM << SEP;
   str1.getChars(0, 2, dest1, 10);     // "ThisIsThe Thirce String?"
   outs << DLM << dest1 << DLM << SEP;
   str1.getChars(24, 24, dest1, 17);   // "ThisIsThe Thirce String?"
   outs << DLM << dest1 << DLM << SEP;
   str1.getChars(23, 23, dest1, 8);    // "ThisIsTh? Thirce String?"
   outs << DLM << dest1 << DLM << SEP;
   str1.getChars(4, 7, dest1, 0);      // "IsThIsTh? Thirce String?"
   outs << DLM << dest1 << DLM << SEP;

   }  // test42()

//************************************************************************
// Tests of class StringBuffer.
//************************************************************************

//------------------------------------------------------------------------
// #1000 StringBuffer constructors (and the destructor).
//------------------------------------------------------------------------
void test1000()
   {
   StringBuffer buf1;                  // ""
   StringBuffer buf2(5);               // "     " (5 spaces)

   String str1 = "A String";
   StringBuffer buf3 = str1;           // "A String"
   StringBuffer buf4 = buf3;           // "A String"

   outs << "#1000" << SEP
        << DLM << buf1 << DLM << SEP << DLM << buf2 << DLM << SEP
        << DLM << buf3 << DLM << SEP << DLM << buf4 << DLM << SEP;

   }  // test1000()

//------------------------------------------------------------------------
// #1001 Assignment and cast to const char * operators.
//------------------------------------------------------------------------
void test1001()
   {
   StringBuffer buf1 = String("the assigned value");
   StringBuffer buf2(20);
   StringBuffer buf3 = buf1;

   buf1 = buf1;               // "the assigned value"
   buf2 = buf1;               // "the assigned value"
   buf1 = buf3;               // "the assigned value"
   const char *arr1 = buf1;   // "the assigned value"

   outs << "#1001" << SEP
        << DLM << buf1 << DLM << SEP
        << DLM << buf2 << DLM << SEP
        << DLM << buf3 << DLM << SEP
        << DLM << arr1 << DLM << SEP;

   }  // test1001()

//------------------------------------------------------------------------
// #1002 capacity(), length() and charAt().
//------------------------------------------------------------------------
void test1002()
   {
   StringBuffer buf1;
   StringBuffer buf2 = String("A non-empty one.");   // length == 16
   StringBuffer buf3(500);

   outs << "#1002" << SEP << buf1.length() << SEP << buf2.length() << SEP
        << DLM << buf2.charAt(0)  << DLM << SEP    // "/A/"
        << DLM << buf2.charAt(1)  << DLM << SEP    // "/ /"
        << DLM << buf2.charAt(12) << DLM << SEP    // "/o/"
        << DLM << buf2.charAt(13) << DLM << SEP    // "/n/"
        << DLM << buf2.charAt(14) << DLM << SEP    // "/e/"
        << DLM << buf2.charAt(15) << DLM << SEP    // "/./"
        << String::valueOf((buf2.charAt(16) == '\0') ? true : false) << SEP;
                                                             // "true"

   // Note: these will change if the formula for determining buffer
   //       capacities changes:
   outs << buf1.capacity() << SEP      // 31
        << buf2.capacity() << SEP      // 16
        << buf3.capacity()<< SEP;      // 511

   }  // test1002()

//------------------------------------------------------------------------
// #1003 ensureCapacity().
//------------------------------------------------------------------------
void test1003()
   {
   StringBuffer buf1 = String("A 22-character string."); // capacity == 22
   const int NUM_TESTS = 6;
   boolean result[NUM_TESTS];

   int newCapacity = 0;
   buf1.ensureCapacity(newCapacity);
   result[0] = (buf1.capacity() >= newCapacity) ? true : false;   // true
   newCapacity = 21;
   buf1.ensureCapacity(newCapacity);
   result[1] = (buf1.capacity() >= newCapacity) ? true : false;   // true
   newCapacity = 22;
   buf1.ensureCapacity(newCapacity);
   result[2] = (buf1.capacity() >= newCapacity) ? true : false;   // true
   newCapacity = 23;
   buf1.ensureCapacity(newCapacity);
   result[3] = (buf1.capacity() >= newCapacity) ? true : false;   // true
   newCapacity = 1025;
   buf1.ensureCapacity(newCapacity);
   result[4] = (buf1.capacity() >= newCapacity) ? true : false;   // true
   newCapacity = 21;
   buf1.ensureCapacity(newCapacity);
   result[5] = (buf1.capacity() >= newCapacity) ? true : false;   // true

   outs << "#1003" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(result[i]) << SEP;
      }
   }  // test1003()

//------------------------------------------------------------------------
// #1004 setCharAt().
//------------------------------------------------------------------------
void test1004()
   {
   StringBuffer buf1 = String("The original buffer");   // length == 19
   const int NUM_TESTS = 7;
   StringBuffer result[NUM_TESTS];

   buf1.setCharAt(0, 'T');                   // "The original buffer"
   result[0] = buf1;
   buf1.setCharAt(18, 'r');                  // "The original buffer"
   result[1] = buf1;
   buf1.setCharAt(0, 't');                   // "the original buffer"
   result[2] = buf1;
   buf1.setCharAt(8, '-');                   // "the orig-nal buffer"
   result[3] = buf1;
   buf1.setCharAt(8, '-');                   // "the orig-nal buffer"
   result[4] = buf1;
   buf1.setCharAt(18, 't');                  // "the orig-nal buffet"
   result[5] = buf1;
   buf1.setCharAt(5, String::terminator());  // "the o"
   result[6] = buf1;

   outs << "#1004" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test1004()

//------------------------------------------------------------------------
// #1005 setLength().
//------------------------------------------------------------------------
void test1005()
   {
   StringBuffer buf1 = String("Mimsy were the ...");   // length == 18
   StringBuffer buf2;
   StringBuffer buf3;
   const int NUM_TESTS = 11;
   StringBuffer result[NUM_TESTS];

   buf1.setLength(18);
   result[0] = buf1;       // "Mimsy were the ..."
   buf1.setLength(19);
   result[1] = buf1;       // "Mimsy were the ... "
   buf1.setLength(24);
   result[2] = buf1;       // "Mimsy were the ...      " (6 trail. spaces)
   buf1.setLength(17);
   result[3] = buf1;       // "Mimsy were the .."
   buf1.setLength(18);
   result[4] = buf1;       // "Mimsy were the .. "
   buf1.setLength(4);
   result[5] = buf1;       // "Mims"
   buf1.setLength(6);
   result[6] = buf1;       // "Mims  "
   buf1.setLength(0);
   result[7] = buf1;       // ""
   buf2.setLength(5);
   result[8] = buf2;       // "     " (5 spaces)
   buf2.setLength(0);
   result[9] = buf2;       // ""
   buf3.setLength(0);
   result[10] = buf3;      // ""

   outs << "#1005" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test1005()

//------------------------------------------------------------------------
// #1006 toString() and getChars().
//------------------------------------------------------------------------
void test1006()
   {
   StringBuffer buf1 = String("ThisIsThe Source String?"); // length == 24
   StringBuffer buf2;

   outs << "#1006" << SEP;
   outs << DLM << buf1.toString() << DLM << SEP  // "ThisIsThe Source String?"
        << DLM << buf2.toString() << DLM << SEP; // ""

   char *dest1 = buf1.toString().toCharArray();
   char *dest2 = buf2.toString().toCharArray();

   buf2.getChars(0, 0, dest1, 0);      // "ThisIsThe Source String?"
   outs << DLM << dest1 << DLM << SEP;
   buf2.getChars(0, 0, dest2, 0);      // ""
   outs << DLM << dest2 << DLM << SEP;
   buf2.getChars(0, 0, dest1, 24);     // "ThisIsThe Source String?"
   outs << DLM << dest1 << DLM << SEP;
   buf1.getChars(0, 2, dest1, 10);     // "ThisIsThe Thirce String?"
   outs << DLM << dest1 << DLM << SEP;
   buf1.getChars(24, 24, dest1, 17);   // "ThisIsThe Thirce String?"
   outs << DLM << dest1 << DLM << SEP;
   buf1.getChars(23, 23, dest1, 8);    // "ThisIsTh? Thirce String?"
   outs << DLM << dest1 << DLM << SEP;
   buf1.getChars(4, 7, dest1, 0);      // "IsThIsTh? Thirce String?"
   outs << DLM << dest1 << DLM << SEP;

   }  // test1006()

//------------------------------------------------------------------------
// #1007 append(String).
//------------------------------------------------------------------------
void test1007()
   {
   String str1 = "Before appending:";   // length == 17
   String str2;
   String str3 = "appendage";
   StringBuffer buf1 = str1;
   StringBuffer buf2;
   const int NUM_TESTS = 7;
   StringBuffer result[NUM_TESTS];

   result[0] = buf1.append(str2);      // "Before appending:"
   result[1] = buf2.append(str2);      // ""
   result[2] = buf1.append(str3);      // "Before appending:appendage"
   result[3] = buf2.append(str3);      // "appendage"
   result[4] = buf1.append(str2);      // "Before appending:appendage"
   result[5] = buf2.append(str2);      // "appendage"

   buf1.setLength(7);                  // "Before " [not output !!!]
   result[6] = buf1.append(str3).append(str1);
                                       // "Before appendageBefore appending:"

   outs << "#1007" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test1007()

//------------------------------------------------------------------------
// #1008 append() - remaining versions.
//------------------------------------------------------------------------
void test1008()
   {
   StringBuffer buf1;
   const int NUM_TESTS = 15;
   StringBuffer result[NUM_TESTS];

   char arr[] = "a message.";
   char ch1 = '\0', ch2 = '%', ch3 = ' ', ch4 = 'a';
   result[0] = buf1.append(arr);          // "a message."
   result[1] = buf1.append(arr, 2, 4);    // "a message.mess"
   result[2] = buf1.append(true);         // "a message.messtrue"
   result[3] = buf1.append(false);        // "a message.messtruefalse"
   result[4] = buf1.append(ch4);          // "a message.messtruefalsea"
   result[5] = buf1.append(ch3);          // "a message.messtruefalsea "
   result[6] = buf1.append(ch2);          // "a message.messtruefalsea %"
   result[7] = buf1.append(ch1);          // "a message.messtruefalsea %"
   result[8] = buf1.append(arr, 6, 4);    // "a message.messtruefalsea %age."

   short          si  = -0;
   unsigned short usi = 110;
   int            in  = -3245;
   unsigned int   ui  = 65535U;
   long           li  = -2000000000L;
   unsigned long  uli = 4123456789L;
   buf1.setLength(0);
   result[9] = buf1.append(si);     // "0"
   result[10] = buf1.append(usi);   // "0110"
   result[11] = buf1.append(in);    // "0110-3245"
   result[12] = buf1.append(ui);    // "0110-324565535"
   result[13] = buf1.append(li);    // "0110-324565535-2000000000"
   result[14] = buf1.append(uli);   // "0110-324565535-20000000004123456789"


   outs << "#1008" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test1008()

//------------------------------------------------------------------------
// #1009 insert(StringInt, String).
//------------------------------------------------------------------------
void test1009()
   {
   String str1 = "Before appending:";   // length == 17
   String str2;
   String str3 = "appendage";
   StringBuffer buf1 = str1;
   StringBuffer buf2;
   const int NUM_TESTS = 9;
   StringBuffer result[NUM_TESTS];

   result[0] = buf1.insert(0, str2);      // "Before appending:"
   result[1] = buf2.insert(0, str2);      // ""
   result[2] = buf1.insert(16, str2);     // "Before appending:"
   result[3] = buf1.insert(17, str2);     // "Before appending:"
   result[4] = buf1.insert(12, str2);     // "Before appending:"
   result[5] = buf1.insert(17, str3);     // "Before appending:appendage"
   result[6] = buf2.insert(0, str3);      // "appendage"
   result[7] = buf2.insert(3, str3);      // "appappendageendage"
   result[8] = buf2.insert(17, str3);     // "appappendageendagappendagee"

   outs << "#1009" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test1009()

//------------------------------------------------------------------------
// #1010 insert() - all other versions.
//------------------------------------------------------------------------
void test1010()
   {
   StringBuffer buf1;
   const int NUM_TESTS = 15;
   StringBuffer result[NUM_TESTS];

   char arr[] = "a message.";
   char ch1 = '\0', ch2 = '%', ch3 = ' ', ch4 = 'a';
   result[0] = buf1.insert(0, arr);    // "a message."
   result[1] = buf1.insert(6, arr);    // "a messa message.age."
   result[2] = buf1.insert(8, true);   // "a messa truemessage.age."
   result[3] = buf1.insert(19, false); // "a messa truemessagefalse.age."
   result[4] = buf1.insert(11, ch1);   // "a messa truemessagefalse.age."
   result[5] = buf1.insert(1, ch2);    // "a% messa truemessagefalse.age."
   result[6] = buf1.insert(7, ch3);    // "a% mess a truemessagefalse.age."
   result[7] = buf1.insert(6, ch4);    // "a% mesas a truemessagefalse.age."

   short          si  = -0;
   unsigned short usi = 110;
   int            in  = -3245;
   unsigned int   ui  = 65535U;
   long           li  = -2000000000L;
   unsigned long  uli = 4123456789L;
   buf1.setLength(0);
   result[8] = buf1.insert(0, in);     // "-3245"
   result[9] = buf1.insert(3, si);     // "-32045"
   result[10] = buf1.insert(5, usi);   // "-32041105"
   result[11] = buf1.insert(2, in);    // "-3-32452041105"
   result[12] = buf1.insert(0, ui);    // "65535-3-32452041105"
   result[13] = buf1.insert(11, li);   // "65535-3-324-200000000052041105"
   result[14] = buf1.insert(3, uli);
                              // "655412345678935-3-324-200000000052041105"

   outs << "#1010" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }

   }  // test1010()

//------------------------------------------------------------------------
// #1011 overlay().
//------------------------------------------------------------------------
void test1011()
   {
   String str1 = "The string to overlay over.";   // length == 27
   String str2;
   String str3 = "layer";
   StringBuffer buf1 = str1;
   StringBuffer buf2;
   const int NUM_TESTS = 9;
   StringBuffer result[NUM_TESTS];

   result[0] = buf1.overlay(27, str2);    // "The string to overlay over."
   result[1] = buf2.overlay(0, str2);     // ""
   result[2] = buf1.overlay(0, str2);     // "The string to overlay over."
   result[3] = buf1.overlay(12, str2);    // "The string to overlay over."
   result[4] = buf1.overlay(18, str3);    // "The string to overlayerver."
   result[5] = buf2.overlay(0, str3);     // "layer"
   result[6] = buf2.overlay(5, str3);     // "layerlayer"
   result[7] = buf1.overlay(0, str3);     // "layertring to overlayerver."
   result[8] = buf2.overlay(9, str3);     // "layerlayelayer"

   outs << "#1011" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << DLM << result[i] << DLM << SEP;   // "/[string]/"
      }
   }  // test1011()

//------------------------------------------------------------------------
// clear().
//------------------------------------------------------------------------
void test1012()
   {
	StringBuffer buf1;
   String *str1 = new String("Fun with bathyspheres...");
   StringBuffer buf2 = *str1;
   delete str1;

   outs << "#1012" << endl;

   // test buffers with only one reference to their data:
   buf1.clear();
   outs << buf1 << endl;       // ""
   buf2.clear();
   outs << buf2 << endl;       // ""

   // test buffers with more than one reference to their data:
   String str2 = "Another whirl";
   StringBuffer buf3(str2);
   buf3.clear();
   outs << buf3 << endl;       // ""

   }  // test1012()

//************************************************************************
// Tests of class WordIterator.
//************************************************************************

//------------------------------------------------------------------------
// #2000 hasMoreWords() and nextWord().
//------------------------------------------------------------------------
void test2000()
   {
   String str1 = "Many, many\tMANY\n \t words to choose  from.";
   String str2;
   WordIterator iter1 = str1;
   WordIterator iter2 = str2;
   const int NUM_TESTS = 12;
   boolean areMore[NUM_TESTS];
   String result[NUM_TESTS];

   areMore[0] = iter2.hasMoreWords();         // false
   result[0] =  iter2.nextWord();             // ""
   areMore[1] = iter2.hasMoreWords();         // false
   result[1] =  iter2.nextWord();             // ""
   areMore[2] = iter1.hasMoreWords();         // true
   result[2] =  iter1.nextWord();             // "Many,"
   areMore[3] = iter1.hasMoreWords();         // true
   result[3] =  iter1.nextWord();             // "many"
   areMore[4] = iter1.hasMoreWords();         // true
   result[4] =  iter1.nextWord();             // "MANY"
   areMore[5] = iter1.hasMoreWords();         // true
   result[5] =  iter1.nextWord();             // "words"
   areMore[6] = iter1.hasMoreWords();         // true
   result[6] =  iter1.nextWord();             // "to"
   areMore[7] = iter1.hasMoreWords();         // true
   result[7] =  iter1.nextWord();             // "choose"
   areMore[8] = iter1.hasMoreWords();         // true
   result[8] =  iter1.nextWord();             // "from."
   areMore[9] = iter1.hasMoreWords();         // false
   result[9] =  iter1.nextWord();             // ""
   areMore[10] = iter1.hasMoreWords();        // false
   result[10] =  iter1.nextWord();            // ""
   areMore[11] = iter1.hasMoreWords();        // false
   result[11] =  iter1.nextWord();            // ""

   outs << "#2000" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(areMore[i]) << SEP
           << DLM << result[i] << DLM << SEP;      // "/[string]/"
      }
   }  // test2000()

//------------------------------------------------------------------------
// #2001 WordIterator constructors and assignment operator.
//------------------------------------------------------------------------
void test2001()
   {
   String str1 = "eight six seven five,\tthree oh nine.";
   String str2;
   WordIterator iter1 = str1;
   WordIterator iter2 = str2;
   WordIterator iter3 = iter1;
   WordIterator iter4 = iter2;
   const int NUM_TESTS = 16;
   boolean areMore[NUM_TESTS];
   String result[NUM_TESTS];

   areMore[0] = iter2.hasMoreWords();        // false
   result[0] =  iter2.nextWord();            // ""
   areMore[1] = iter4.hasMoreWords();        // false
   result[1] =  iter4.nextWord();            // ""

   areMore[2] = iter1.hasMoreWords();        // true
   result[2] =  iter1.nextWord();            // "eight"
   areMore[3] = iter1.hasMoreWords();        // true
   result[3] =  iter1.nextWord();            // "six"
   areMore[4] = iter1.hasMoreWords();        // true
   result[4] =  iter1.nextWord();            // "seven"
   areMore[5] = iter1.hasMoreWords();        // true
   result[5] =  iter1.nextWord();            // "five,"
   areMore[6] = iter3.hasMoreWords();        // true
   result[6] =  iter3.nextWord();            // "eight"

   iter4 = iter1;
   areMore[7] = iter3.hasMoreWords();        // true
   result[7] =  iter3.nextWord();            // "six"
   areMore[8] = iter4.hasMoreWords();        // true
   result[8] =  iter4.nextWord();            // "three"
   areMore[9] = iter4.hasMoreWords();        // true
   result[9] =  iter4.nextWord();            // "oh"
   areMore[10] = iter4.hasMoreWords();       // true
   result[10] =  iter4.nextWord();           // "nine."
   areMore[11] = iter4.hasMoreWords();       // false
   result[11]  = iter4.nextWord();           // ""
   areMore[12] = iter1.hasMoreWords();       // true
   result[12]  = iter1.nextWord();           // "three"

   iter1 = iter1;
   areMore[13] = iter1.hasMoreWords();       // true
   result[13]  = iter1.nextWord();           // "oh"

   iter1 = iter2;
   areMore[14] = iter1.hasMoreWords();       // false
   result[14]  = iter1.nextWord();           // ""

   iter4 = iter4;
   areMore[15] = iter4.hasMoreWords();       // false
   result[15]  = iter4.nextWord();           // ""

   outs << "#2001" << SEP;
   for (int i = 0; i < NUM_TESTS; i++)
      {
      outs << String::valueOf(areMore[i]) << SEP
           << DLM << result[i] << DLM << SEP;      // "/[string]/"
      }
   }  // test2001()

